@@ -5,7 +5,7 @@
 # This script automates creating universal binaries of LyX on Mac.
 # Author: Bennett Helm (and extended by Konrad Hofbauer)
 # latest changes by Stephan Witt
-# Last modified: July 2014
+# Last modified: February 2020
 
 QtAPI=${QtAPI:-"-cocoa"}
 QtVersion=${QtVersion:-"4.6.3"}
@@ -152,6 +152,7 @@ usage() {
 	echo " --with-build-path=PATH ....." default \${lyx-src-dir}/../lyx-build
 	echo " --with-dmg-location=PATH ..." default \${build-path}
 	echo " --with-binary-strip=yes ...." default no
+	echo " --codesign-identity=CSID ..." default is w/o CSID
 	echo
 	echo "All other arguments with -- are passed to configure"
 	echo "including the defaults: ${LyXConfigureOptions}"
@@ -212,6 +213,10 @@ while [ $# -gt 0 ]; do
 		esac
 		shift
 		;;
+	--codesign-identity=*)
+		CODESIGN_IDENTITY=$(echo "${1}"|cut -d= -f2)
+		shift
+		;;
 	--libmagic-deployment=*)
 		libmagic_deployment=$(echo ${1}|cut -d= -f2)
 		shift
@@ -625,10 +630,14 @@ framework_name() {
 LYX_FILE_LIST="lyx lyxclient tex2lyx lyxconvert"
 BUNDLE_PATH="Contents/MacOS"
 LYX_BUNDLE_PATH="${LyxAppPrefix}/${BUNDLE_PATH}"
+
 build_lyx() {
 	# Clear Output
 	if [ -n "${LyxAppZip}" -a -f "${LyxAppZip}" ]; then rm "${LyxAppZip}"; fi
-	if [ -d "${LyxAppPrefix}" ]; then rm -rf "${LyxAppPrefix}"; fi
+	if [ -d "${LyxAppPrefix}" ]; then
+		find "${LyxAppPrefix}" -type d -exec chmod u+w '{}' \;
+		rm -rf "${LyxAppPrefix}"
+	fi
 
 	case "${EnableCXX11}" in
 	"--enable-cxx11")
@@ -724,17 +733,37 @@ installname() {
 }
 
 private_framework() {
-	fwdir=$(framework_name "$1")
-	source="$2"
-	target="$3"
+	fwname="$1" ; shift
+	source="$1" ; shift
+	target="$1" ; shift
+	version=$(echo ${1:-"1.1.1"}.1.1.1 | cut -d. -f1-3) ; shift
+	fwdir=$(framework_name "${fwname}")
 	condir=$(content_directory "${target}")
 	libnm=$(basename "${source}")
-	mkdir -p "${condir}/${fwdir}"
-	if [ ! -f "${condir}/${fwdir}/${libnm}" ]; then
-		cp -p "${source}" "${condir}/${fwdir}"
-		installname -id "@executable_path/../${fwdir}/${libnm}" "${condir}/${fwdir}/${libnm}"
+	libid="org.lyx."$(echo "${libnm}" | cut -d. -f1)
+	svrsn=$(echo "${version}" | cut -d. -f1-2)
+	fwvrsn="1"
+	mkdir -p "${condir}/${fwdir}"/Versions/${fwvrsn}/Headers
+	mkdir -p "${condir}/${fwdir}"/Versions/${fwvrsn}/Resources
+	if [ ! -f "${condir}/${fwdir}/Versions/${fwvrsn}/${fwname}" ]; then
+		cp -p "${source}" "${condir}/${fwdir}/Versions/${fwvrsn}/${fwname}"
+		for hfile in "$@" ; do
+			test -f "${hfile}" && cp -p "${hfile}" "${condir}/${fwdir}"/Versions/${fwvrsn}/Headers
+		done
+		ln -s ${fwvrsn} "${condir}/${fwdir}/Versions/Current"
+		ln -s Versions/Current/Headers "${condir}/${fwdir}/Headers"
+		ln -s Versions/Current/Resources "${condir}/${fwdir}/Resources"
+		ln -s Versions/Current/"${fwname}" "${condir}/${fwdir}/${fwname}"
+		installname -id "@executable_path/../${fwdir}/${fwname}" "${condir}/${fwdir}/${fwname}"
+		if [ -f "${LyxSourceDir}"/development/LyX-Mac-frameworks-template.plist ]; then
+			cat "${LyxSourceDir}"/development/LyX-Mac-frameworks-template.plist | sed \
+				-e "s/@CFBundleExecutable@/${fwname}/" \
+				-e "s/@CFBundleIdentifier@/${libid}/" \
+				-e "s/@CFBundleShortVersionString@/${svrsn}/" \
+				-e "s/@CFBundleVersion@/${version}/" > "${condir}/${fwdir}"/Resources/Info.plist
+		fi
 	fi
-	installname -change "${source}" "@executable_path/../${fwdir}/${libnm}" "${target}"
+	installname -change "${source}" "@executable_path/../${fwdir}/${fwname}" "${target}"
 }
 
 deploy_qtlibs() {
@@ -767,7 +796,9 @@ EOF
 		test -d "${condir}/${fwdir}" || (
 			echo Copy framework "${source}/lib/"$(basename "${fwdir}")
 			cp -pR "${source}/lib/"$(basename "${fwdir}") "${condir}/${fwdir}"
-			rm -f "${condir}/${fwdir}/${version}${libnm}"_debug
+			rm -f "${condir}/${fwdir}/${libnm}"_debug "${condir}/${fwdir}/${version}${libnm}"_debug
+			test -f "${condir}/${fwdir}/${libnm}".prl && mv "${condir}/${fwdir}/${libnm}".prl "${condir}/${fwdir}"/Resources
+			test -f "${condir}/${fwdir}/${libnm}"_debug.prl && mv "${condir}/${fwdir}/${libnm}"_debug.prl "${condir}/${fwdir}"/Resources
 			installname -id "@executable_path/../${fwdir}/${version}${libnm}" "${condir}/${fwdir}/${version}${libnm}"
 			find "${condir}/PlugIns" "${condir}/"$(dirname "${fwdir}") -name Headers -prune -o -type f -print | while read filename ; do
 				if [ "${filename}" != "${target}" ]; then
@@ -818,13 +849,16 @@ convert_universal() {
 			lipo -create ${OBJ_LIST} -o "${BUNDLE_PATH}/${file}"
 		fi
 		if [ -f "${LibMagicInstallDir}/lib/${LibMagicLibrary}" -a "yes" = "${libmagic_deployment}" ]; then
-			private_framework LibMagic "${LibMagicInstallDir}/lib/${LibMagicLibrary}" "${LYX_BUNDLE_PATH}/${file}"
+			private_framework LibMagic "${LibMagicInstallDir}/lib/${LibMagicLibrary}" "${LYX_BUNDLE_PATH}/${file}" \
+				"${LibMagicVersion}" "${LibMagicInstallHdr}"
 		fi
 		if [ -f "${ASpellInstallDir}/lib/${ASpellLibrary}" -a "yes" = "${aspell_deployment}" ]; then
-			private_framework Aspell "${ASpellInstallDir}/lib/${ASpellLibrary}" "${LYX_BUNDLE_PATH}/${file}"
+			private_framework Aspell "${ASpellInstallDir}/lib/${ASpellLibrary}" "${LYX_BUNDLE_PATH}/${file}" \
+				"${ASpellVersion}" "${ASpellInstallHdr}"
 		fi
 		if [ -f "${HunSpellInstallDir}/lib/${HunSpellLibrary}" -a "yes" = "${hunspell_deployment}" ]; then
-			private_framework Hunspell "${HunSpellInstallDir}/lib/${HunSpellLibrary}" "${LYX_BUNDLE_PATH}/${file}"
+			private_framework Hunspell "${HunSpellInstallDir}/lib/${HunSpellLibrary}" "${LYX_BUNDLE_PATH}/${file}" \
+				"${HunSpellVersion}" "${HunSpellInstallDir}/include/hunspell/"*.hxx "${HunSpellInstallHdr}"
 		fi
 		if [ -d "${QtInstallDir}/lib/QtCore.framework/Versions/${QtFrameworkVersion}" -a "yes" = "${qt_deployment}" ]; then
 			deploy_qtlibs "${LYX_BUNDLE_PATH}/${file}"
@@ -838,10 +872,43 @@ convert_universal() {
 		done
 	done
 	for arch in ${ARCH_LIST} ; do
-		rm -f ${BUNDLE_PATH}/*-${arch}
+		rm -f "${BUNDLE_PATH}"/*-${arch}
 	done
 }
 
+# -------------------------
+# Create code sign signatures
+# -------------------------
+code_sign() {
+	target="$1"
+	condir=$(content_directory "${target}"/lyx)
+	appdir=$(dirname "${condir}")
+	# have to sign frameworks first
+	for fwname in Aspell Hunspell LibMagic ; do
+		fwitem="${condir}"/$(framework_name "${fwname}")
+		if [ -d "${fwitem}" ]; then
+			codesign --verbose --force --sign "${CODESIGN_IDENTITY}" "${fwitem}"
+		fi
+	done
+	for csitem in \
+		"${condir}"/Frameworks/Qt*.framework/Versions/${QtFrameworkVersion} \
+		"${condir}"/PlugIns/*/lib*.dylib \
+		"${condir}"/Library/Spotlight/* \
+		"${target}"/inkscape \
+		"${target}"/maxima \
+		"${target}"/tex2lyx \
+		"${target}"/lyxeditor \
+		"${target}"/lyxconvert \
+		"${target}"/lyxclient
+	do
+		codesign --verbose --force --sign "${CODESIGN_IDENTITY}" "${csitem}"
+	done
+
+	/usr/bin/codesign --verbose --force --sign "${CODESIGN_IDENTITY}" "${appdir}" || {
+		echo Warning: codesign failed with certificate named '"'${CODESIGN_IDENTITY}'"'
+	}
+}
+
 deduplicate() {
 	find "$@" -type f -print | while read file ; do
 		echo $(md5 -q "$file") "$file"
@@ -950,7 +1017,7 @@ make_dmg() {
 	VOLUME=$(mount |grep ${DEVICE} | cut -f 3 -d ' ')
 
 	# copy in the application bundle
-	ditto --hfsCompression "${LyxAppDir}.app" "${VOLUME}/${LyxName}.app"
+	ditto --hfsCompression "${LyxAppPrefix}" "${VOLUME}/${LyxName}.app"
 
 	# copy in background image
 	mkdir -p "${VOLUME}/Pictures"
@@ -1003,5 +1070,7 @@ if [ ${LyxOnlyPackage:-"no"} = "no" ]; then
 	build_lyx
 	convert_universal
 	copy_dictionaries
+	test -n "${CODESIGN_IDENTITY}" && code_sign "${LYX_BUNDLE_PATH}"
+	find "${LyxAppPrefix}" -type d -exec chmod a-w '{}' \;
 fi
 build_package
