@@ -25,6 +25,7 @@ class CmdDef;
 class Converters;
 class DispatchResult;
 class ErrorItem;
+class Formats;
 class FuncRequest;
 class FuncStatus;
 class KeyMap;
@@ -51,8 +52,11 @@ enum OverwriteFiles {
 };
 
 extern bool use_gui;
+extern bool verbose;
+extern bool ignore_missing_glyphs;
 extern RunMode run_mode;
 extern OverwriteFiles force_overwrite;
+extern double qt_scale_factor;
 
 namespace frontend {
 class Application;
@@ -136,6 +140,8 @@ private:
 	friend ServerSocket & theServerSocket();
 	friend Converters & theConverters();
 	friend Converters & theSystemConverters();
+	friend Formats & theFormats();
+	friend Formats & theSystemFormats();
 	friend Messages const & getMessages(std::string const & language);
 	friend Messages const & getGuiMessages();
 	friend KeyMap & theTopLevelKeymap();
