@@ -52,22 +52,4 @@ int InsetMarginal::docbook(odocstream & os,
 }
 
 
-void InsetMarginal::addToToc(DocIterator const & cpit, bool output_active,
-				  UpdateType utype) const
-{
-	DocIterator pit = cpit;
-	pit.push_back(CursorSlice(const_cast<InsetMarginal &>(*this)));
-
-	docstring tooltip;
-	text().forOutliner(tooltip, TOC_ENTRY_LENGTH);
-	docstring const str = tooltip;
-	tooltip = support::wrapParas(tooltip, 0, 60, 2);
-	
-	shared_ptr<Toc> toc = buffer().tocBackend().toc("marginalnote");
-	toc->push_back(TocItem(pit, 0, str, output_active, tooltip));
-
-	// Proceed with the rest of the inset.
-	InsetFootlike::addToToc(cpit, output_active, utype);
-}
-
 } // namespace lyx
